/**\file
 * \brief Defines tags for the stellar zone.
 *
 * \ingroup Utilities_group
 */

#ifndef __STELLAR_ZONE_H
#define __STELLAR_ZONE_H

///Tags for the stellar zone
enum StellarZone {total=0, ///< The entire star
	core=1, ///< The radiative core
	radiative=1, ///< The radiative core
	envelope=2, ///< The convective envelope
	convective=2 ///< The convective envelope
};

#endif
